

#include "sa.h"

//-----------------------------------------------------------------------------------
// Frequency display bands
//
//   2048 point FFT (1024 frequency bins)
//   @ 50 KHz sampling rate
//   each bin = 50 KHz / 2048 = 24.4140625 Hz
//-----------------------------------------------------------------------------------

const int freqBin[16] __attribute__ ((space(auto_psv), aligned(2))) =
{
   // the 1'st frequency cell [0] is 0-12 Hz (ignored)
   //
   //         Bin Center Freq Freq Span (Hz)
   //         --- ----------- -------------- 
       1,  //   1     24 Hz      12-37
       1,  //   2     49 Hz      37-61
       1,  //   3     73 Hz      61-85
       1,  //   4     98 Hz      85-110
       1,  //   5    122 Hz     110-134
       2,  //   6    159 Hz     134-183
       3,  //   7    220 Hz     183-256
       5,  //   8    317 Hz     256-378
       9,  //   9    488 Hz     378-598
      15,  //  10    781 Hz     598-964
      25,  //  11   1270 Hz     964-1575
      50,  //  12   2185 Hz    1575-2795
      80,  //  13   3772 Hz    2795-4749
     120,  //  14   6213 Hz    4749-7678
     180,  //  15   9875 Hz    7678-12073
     529   //  16  16036 Hz   12073-20000+ (actual limit is ~ 22 KHz)
  // ====
  // 1023 bins total
};


//-----------------------------------------------------------------------------------
// Physical column scan order for the LED arrays
//-----------------------------------------------------------------------------------

extern volatile unsigned char disp_scan[8]; // physical column scan order

// loaded into RAM for fast access (and without requiring PSV) in interrupt handlers

void initialize_disp_scan()
{
#if PCB_TYPE == PCB_SMALL
#define LED_SCAN_SET
    // display scan is backwards (row 1 is on the rightmost side)
    disp_scan[0] = 0x20;
    disp_scan[1] = 0x40;
    disp_scan[2] = 0x80;
    disp_scan[3] = 0x10;
    disp_scan[4] = 0x04;
    disp_scan[5] = 0x02;
    disp_scan[6] = 0x01;
    disp_scan[7] = 0x08;
#endif

#if PCB_TYPE == PCB_MED
#define LED_SCAN_SET
    // display scan is forwards (row 1 is on the leftmost side)
    disp_scan[0] = 0x20;
    disp_scan[1] = 0x10;
    disp_scan[2] = 0x40;
    disp_scan[3] = 0x80;
    disp_scan[4] = 0x01;
    disp_scan[5] = 0x02;
    disp_scan[6] = 0x04;
    disp_scan[7] = 0x08;
#endif

#ifndef LED_SCAN_SET
#error "LED scan is not supported by this code"
#endif
}


//-----------------------------------------------------------------------------------
// Physical pixel charsets for signal level values
//-----------------------------------------------------------------------------------

#define COLOR_VU  0
#define COLOR_GRN 1
#define COLOR_YEL 2
#define COLOR_RED 3

extern volatile unsigned long charset_mask_bar[17];
extern volatile unsigned long charset_mask_dot[17];

void initialize_charset_bar_VU(volatile unsigned long *p);
void initialize_charset_bar_GRN(volatile unsigned long *p);
void initialize_charset_bar_RED(volatile unsigned long *p);
void initialize_charset_dot_VU(volatile unsigned long *p);
void initialize_charset_dot_GRN(volatile unsigned long *p);
void initialize_charset_dot_RED(volatile unsigned long *p);

void initialize_bar_charset( volatile unsigned long *pCharset, unsigned int zColor )
{
    int i;
    volatile unsigned long *p;

    for (i=0, p = pCharset; i < 17; i++, p++) *p = 0;

    switch (zColor)
    {
        case COLOR_VU:
            initialize_charset_bar_VU( pCharset );
            break;

        case COLOR_GRN:
            initialize_charset_bar_GRN( pCharset );
            break;

        case COLOR_RED:
            initialize_charset_bar_RED( pCharset );
            break;

        case COLOR_YEL:
            initialize_charset_bar_GRN( pCharset );
            initialize_charset_bar_RED( pCharset );
    }
}

void initialize_dot_charset( volatile unsigned long *pCharset, unsigned int zColor )
{
    int i;
    volatile unsigned long *p;

    for (i=0, p = pCharset; i < 17; i++, p++) *p = 0;

    switch (zColor)
    {
        case COLOR_VU:
            initialize_charset_dot_VU( pCharset );
            break;

        case COLOR_GRN:
            initialize_charset_dot_GRN( pCharset );
            break;

        case COLOR_RED:
            initialize_charset_dot_RED( pCharset );
            break;

        case COLOR_YEL:
            initialize_charset_dot_GRN( pCharset );
            initialize_charset_dot_RED( pCharset );
    }
}

void initialize_charset_masks()
{
    int i;
    volatile unsigned long *p;

    for (i=0, p = &charset_mask_bar[0]; i < 17; i++, p++) *p = 0;
    initialize_charset_bar_GRN( &charset_mask_bar[0] );
    initialize_charset_bar_RED( &charset_mask_bar[0] );
    for (i=0, p = &charset_mask_bar[0]; i < 17; i++, p++) *p ^= 0xFFFFFFFF;

    for (i=0, p = &charset_mask_dot[0]; i < 17; i++, p++) *p = 0;
    initialize_charset_dot_GRN( &charset_mask_dot[0] );
    initialize_charset_dot_RED( &charset_mask_dot[0] );
    for (i=0, p = &charset_mask_dot[0]; i < 17; i++, p++) *p ^= 0xFFFFFFFF;
}

// charset bitmapz

#if PCB_TYPE == PCB_SMALL
#define LED_CHARSET_SET
#include "sa_charset_s.c"
#endif

#if PCB_TYPE == PCB_MED
#define LED_CHARSET_SET
#include "sa_charset_m.c"
#endif

#ifndef LED_CHARSET_SET
#error "LED charset is not supported by this code"
#endif
