;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; stock Microchip library from C30 v3.12
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

	.section .libdsp, code

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; _BitReverseComplex: Complex (in-place) Bit Reverse re-organization.
;
; Input:
;	w0 = number stages in FFT (log2NVal)
;	w1 = ptr to complex source vector
;
; System resources usage:
;	{w0..w7}	used, not restored
;	 MODCON		saved, used, restored
;	 XBREV		saved, used, restored
;
; DO and REPEAT instruction usage.
;	1 level DO intruction
;	no REPEAT intructions
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

	.global	_BitReverseComplex

_BitReverseComplex:

;............................................................................

	push	MODCON
	push	XBREV

;............................................................................

	; Prepare for bit reverse addressing
	mov	#0x0200,w2			; uses w2 for bit reverse
	mov	w2,MODCON
	mov	#0x8000,w2			; enable bit reverse addresing
	mov	#0x1,w3				; shifting base
	sl	w3,w0,w3			; w3 = N = 2^(log2NVal)
	ior	w3,w2,w2
	mov	w2,XBREV			; pivot N (equivalent to a pivot N/2 for complex data)

;............................................................................

	; Bit reversal
	mov	w1,w2				; w2: bit reverse addressing
					    	; w1: sequential addressing
	mov	#2,w6				; w6 = sizeof(fractional)
	mov	#4,w7				; w7 = sizeof(fractcomplex)

	dec	w3,w3				; w3 = N-1
	do	w3,_endBitRev		; {	; do (N-1)+1 times
	cp	w2,w1				; w2-w1 ?
	bra	le,_skip			; w2 <= w1 => skip to next
	; Swap complex value pairs
	mov.d	[w1],w4			; w4 = real[n], w5 = imag[n]
	mov	[w2],[w1]		    ; replace real part sequential
	mov	[w2+w6],[w1+w6]		; replace imag part sequential
	mov.d	w4,[w2]			; replace real/imag bit reverse
_skip:
	add	w1,w7,w1			; w1 += sizeof(fractcomplex) sequential update
_endBitRev: ; loop execution ends at the FOLLOWING INSTRUCTION...
	mov	[w2],[w2++]			; bit reverse update (only on data writes)

;............................................................................

    pop     XBREV
    pop     MODCON

;............................................................................

	return

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

	.end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
