;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

    .include "sa.inc" ; hardware registers for particular CPU chip being used

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

	.section .libdsp, code

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; _Complex_Max: Find largest squared magnitude in complex vector
;
; Input:
;   w0 = number of elements in array
;	w1 = address of ptr to complex data array (fractcomplex type)
;
; Return:
;	w0 = maximum value
;   pointer = updated to point to next complex element
;
; System resources usage:
;	{w0..w7}	used, not restored
;	 AccuA		used, not restored
;	 CORCON		saved, used, restored
;
; DO and REPEAT instruction usage.
;	no DO intructions
;	no REPEAT intructions
;
;............................................................................

 .global _Complex_Max

_Complex_Max:

;............................................................................

	; Prepare CORCON for fractional computation
    ; + enable saturation on both accumulators
    ; + enable data space write saturation
    ; + normal saturation (1:31)
    ; + unbiased rounding
    ; + fractional multiply mode
    push	CORCON
    mov     #((1<<SATA)|(1<<SATB)|(1<<SATDW)), w7
    mov     w7, CORCON
 
;............................................................................

    mov     [w1],w2     ; get ptr to complex data vector
    mov     w0,w3       ; get loop count
    clr     w0          ; initialize return value

_DO:
    mov.d   [w2++], w4  ; w4 = REX, w5 = IMX

    mpy     w4*w4, a    ; calculate REX^2 + IMX^2
    mac     w5*w5, a
    sac.r   a, w7

    cpsgt   w0,w7       ; keep maximum value
    mov     w7,w0

    dec     w3,w3
    bra     nz, _DO

    ; update data pointer
    mov     w2, [w1]

;............................................................................

    pop     CORCON

;............................................................................

    return

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

	.end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
